% This function calculates the s-AUC and s-NSS with the selected video database with a fixed structure detailed in the
% web page: https://wiki.citius.usc.es/inv:downloadable_results:citius_video_database
%
% INPUTS: 
%       Folder with the saliency maps of some videos procesed with some algorithm
%       Folder with the path of the fixation database
%       Selected Eye (left/right)
%
% Try with this EXAMPLE:
% video=ROCVideosLocal('..\Base_FILES\Test\','..\SampleDB\EyeTrackVDB_Sample.mat','Iz');
%
function video=ROCVideosLocal(smaps_dir,nome_eyeTrackData,OjoSEL)

    %%%%%%%%%%%%%%%%%% INICIALIZATION %%%%%%%%%%%%%%%%%%%%
    bModoDEBUG=false;
    % Number of bootstrap iteration and the number of permutation test interations
    nIterBoot=500;           % Def(500)
    nIterPermTest=2000;      % Def(2000)
    clc;

    % Selection of the metrics ...
    bUseROC=true;
    bUseNSS=true;  
    
    % Threshold for the AUC curve...
    thres=1:256;

    % Saliency maps suffixes for the evaluated algorithm
    sufx_nome='_sm.avi';

    % Compensate or not the True positives (TP) with the same number of false positives (FP)
    CompensarFPconTP=true;

    % Read the fixation information from the fixation database structure.
    EyeTrackVDB=[];
    if ~exist(nome_eyeTrackData, 'file')
        error(['The file does not exist: ',nome_eyeTrackData]);
    else
        load(nome_eyeTrackData);
    end  
    
    % Check the existence of the saliency maps folder.
    if ~exist(smaps_dir, 'file')
        error(['The folder with the saliency videos does not exist: ',smaps_dir]);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Iterate over all the videos included in the video DB.
    numVideosBD=numel(EyeTrackVDB.Video);
    for ivideo=1:numVideosBD % VIDEOS

        fprintf('\nProcessing video (%2d/%2d) %s',ivideo,numVideosBD,EyeTrackVDB.Video(ivideo).Name);

        % Read the video saliency maps ...
        nomeSeqMapa = [smaps_dir,strtok(EyeTrackVDB.Video(ivideo).Name,'.'),sufx_nome];
        if ~exist(nomeSeqMapa, 'file')
            error(['The file does not exist: ',nomeSeqMapa]);
        else
            if (exist('VideoReader')~=0) %#ok<EXIST>
                try smv = VideoReader(nomeSeqMapa); catch exception, error('VideoReader does not work with this matlab'); end
            elseif (exist('mmreader')~=0) %#ok<EXIST>
                try smv = mmreader(nomeSeqMapa); catch exception, error('mmreader does not work with this matlab'); end %#ok<TNMLP>
            end
        end

        % Check that the number of video frames of the original file and the video saliency are the same
        if(EyeTrackVDB.Video(ivideo).NF~=smv.NumberOfFrames)
           disp(' Warning. The number of frames of this video does not match the number in the database');
        end

        % For each video ...
        for iframe=1:smv.NumberOfFrames  %FRAMES

            fprintf('.');
            % Inicializacion variables
            if(bUseROC),vROC=zeros(nIterBoot,1);end
            if(bUseNSS),vNSS=zeros(nIterBoot,1);end         
            sfixes=zeros(256,1);          

            % Para o test de perm. necesitamos as saliencias (os datos, non as distribucins)
            Sal_En_TP=[];
            Sal_En_FP=[];

            % Os mapas dos algoritmos deben ser enteiros entre [0,255] e ter unha dimensins de 240x320 (rango fixacions humanos)
            tmp=double(rgb2gray(read(smv,iframe)));         % Se lee y se convierte a un SOLO canal de niveles de gris.
            smap = imresize(tmp, [240 320], 'bilinear');	% Se reescala la imagen ...
            smap=uint8(floor(mat2gray(smap)*255));          % Se equaliza el histograma, ya que no se fijan los parmetros [min max] de la funcin mat2gray

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Achamos o histograma dos TP (tomanse o valor do mapa de saliencia do algoritmo sobre as posicions das
            % fixacions dos humanos como a resposta correcta. (non varia coas iteracions para un frame dado).
            if(strcmpi(OjoSEL,'Iz')), fixes=ceil(EyeTrackVDB.Video(ivideo).Frame(iframe).FixL(:,1:2));
            else fixes=ceil(EyeTrackVDB.Video(ivideo).Frame(iframe).FixR(:,1:2)); end
            for j=1:size(fixes,1)
                valSAL_XY=smap(fixes(j,2)+1,fixes(j,1)+1)+1;    % MODIFICADO, ya que las FIX estn en el rango [0 WDTH-1]X[0 HEIGH-1]
                sfixes(valSAL_XY)=sfixes(valSAL_XY)+1;
                Sal_En_TP(j)=valSAL_XY; %#ok<AGROW>
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            % Si no hay fijaciones durante ese fotograma ya no se hace el bootstrap
            if(isempty(fixes))
                if(bUseROC),vROC=NaN;end
                if(bUseNSS),vNSS=NaN;end             
            else
                % A las mtricas que emplean las pos de los TP y FP se les aplica Bootstrap con diferentes FP
                for iter=1:nIterBoot   % bootstrap 

                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    % Achamos o histograma dos FP, tomanse o valor do mapa de saliencia do algoritmo sobre as posicions das
                    % fixacions baralladas dos humanos como a resposta incorrecta (eliximos ao azar un video e un frame
                    % (primeira carta) para obter as fixacions baralladas (vara en cada iteracin para un frame dado)
                    sfixes_shuffle=zeros(256,1);
                    Sal_En_FP=[];
                    % Barallamos os videos restantes e elixo a primeira carta
                    indVideos=[1:ivideo-1 ivideo+1:numel(EyeTrackVDB.Video)];
                    permIND=randperm(numel(EyeTrackVDB.Video)-1);
                    idv=indVideos(permIND);
                    % Barallamos os frames do video elexido
                    idf=randperm(EyeTrackVDB.Video(idv(1)).NF);

                    % En funcin do parmetro faise a comprobacin do nmero de frames ...
                    if(CompensarFPconTP)
                        % Acumulo fixacins ata conseguir tantas coma as TP
                        fixes_shuffle=[];nIterTMP=1;
                        while ((size(fixes_shuffle,1)<sum(sfixes))||(nIterTMP>size(idf,2)))
                            if(strcmpi(OjoSEL,'Iz')), 
                                fix_RAND=EyeTrackVDB.Video(idv(1)).Frame(idf(nIterTMP)).FixL;
                                fixes_shuffle=[fixes_shuffle;ceil(fix_RAND(:,1:2))];  %#ok<AGROW>
                            else
                                fix_RAND=EyeTrackVDB.Video(idv(1)).Frame(idf(nIterTMP)).FixR;
                                fixes_shuffle=[fixes_shuffle;ceil(fix_RAND(:,1:2))]; %#ok<AGROW>
                            end 
                            nIterTMP=nIterTMP+1;
                        end
                    else
                        % Se toman las fijaciones de un frame aleatorio 
                        if(strcmpi(OjoSEL,'Iz')),
                            fix_RAND=EyeTrackVDB.Video(idv(1)).Frame(idf(1)).FixL;
                            fixes_shuffle=ceil(fix_RAND(:,1:2));
                        else
                            fix_RAND=EyeTrackVDB.Video(idv(1)).Frame(idf(1)).FixR;
                            fixes_shuffle=ceil(fix_RAND(:,1:2));
                        end
                    end

                    % Creamos a distribucin para os FP usando o valor de saliencia nos puntos das fixacions
                    for j=1:size(fixes_shuffle,1)
                        valSAL_XY=smap(fixes_shuffle(j,2)+1,fixes_shuffle(j,1)+1)+1; % MODIFICADO, ya que las FIX estn en el rango [0 WDTH-1]X[0 HEIGH-1]
                        sfixes_shuffle(valSAL_XY)=sfixes_shuffle(valSAL_XY)+1;
                        Sal_En_FP(j)=valSAL_XY; %#ok<AGROW>
                    end
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                    % Si estamos en modo depuracin se muestran las posiciones de TP y FP
                    if(bModoDEBUG&&(mod(iter,50)==1))
                        figure(1);clf;imshow(smap);axis equal;axis tight;hold on;
                        plot(fixes(:,1),fixes(:,2),'og');
                        plot(fixes_shuffle(:,1),fixes_shuffle(:,2),'xr');
                        title(sprintf('Fotograma %d',iframe));drawnow();pause(0.1);
                    end

                    % ROC curve (Equivalente a la funcin getPuntoROC(Sal_En_FP,Sal_En_TP))
                    if(bUseROC),
                        FP=zeros(length(thres),1);
                        TP=zeros(length(thres),1);
                        for i=1:length(thres)
                            TP(i)=sum(sfixes(thres(i):end))/sum(sfixes);
                            FP(i)=sum(sfixes_shuffle(thres(i):end))/sum(sfixes_shuffle);
                        end
                        rocarea=0;
                        for i=1:length(thres)-1
                            rocarea=rocarea+(FP(i)-FP(i+1))*(TP(i+1)+TP(i))/2;
                        end
                        vROC(iter)=rocarea;
                    end                 

                    % NSS index
                    if(bUseNSS),
                        vNSS(iter) = getNSS(sfixes,sfixes_shuffle);
                    end

                end % Fin do bootstrap
            end

            % Results for this vdeo, on each iteration and the mean values 
            video(ivideo).Name=EyeTrackVDB.Video(ivideo).Name; %#ok<*AGROW>
            video(ivideo).SelEye=OjoSEL; %#ok<AGROW>        

            %% ROC metric
            if(bUseROC),            
                selOkVal=logical(~isnan(vROC));
                video(ivideo).frameAUC(iframe) = mean(vROC(selOkVal));
                video(ivideo).stdAUC(iframe) = std(vROC(selOkVal));
                [~,conflimsBCA] = getBCALimits(vROC(selOkVal));
                video(ivideo).bcaAUC(iframe,:) = conflimsBCA;

                % Test de permutacin para obter os intervalos de confianza dos datos da ROC. Los TP y FP son los
                % correspondientes a la ltima iteracin del loop de bootstrap o a una aleatoria si se emplea parfor.
                rPermuteTest = RandomPermutationROC(Sal_En_TP, Sal_En_FP, nIterPermTest);
                video(ivideo).frameAUC_PTest(iframe,:)=rPermuteTest;
            end

            %% NSS metric 
            if(bUseNSS), 
                selOkVal=logical(~isnan(vNSS));
                video(ivideo).frameNSS(iframe) = mean(vNSS(selOkVal));
                video(ivideo).stdNSS(iframe) = std(vNSS(selOkVal));
                [~,conflimsBCA] = getBCALimits(vNSS(selOkVal));
                video(ivideo).bcaNSS(iframe,:) = conflimsBCA;

                % Test de permutacin para obter os intervalos de confianza dos datos da NSS. Los TP y FP son los
                % correspondientes a la ltima iteracin del loop de bootstrap o a una aleatoria si se emplea parfor.
                rPermuteTest = RandomPermutationNSS(Sal_En_TP, Sal_En_FP, nIterPermTest);
                video(ivideo).frameNSS_PTest(iframe,:)=rPermuteTest;  
            end
            
        end % Remato os frames do video

    end % Fin de videos
    fprintf('\nOk\n');

end


